@@ -21,9 +21,18 @@ import time
 from hashlib import sha1
 from email.utils import formatdate
 
-import zope.location
+try:
+    # Python 3.3+: OS-independent CPU counter
+    from time import process_time
+except ImportError: # pragma: no cover
+    # Python 2.7
+    from time import clock as process_time
+
+
 from persistent import Persistent
-from zope import schema, component
+import zope.location
+from zope import schema
+from zope import component
 from zope.interface import implementer
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.publisher.interfaces.http import IHTTPApplicationRequest
@@ -40,63 +49,67 @@ class MissingClientIdException(Exception):
     """No ClientId found in Request"""
 
 class ICookieClientIdManager(IClientIdManager):
-    """Manages sessions using a cookie"""
+    """
+    Manages client identification using a cookie.
+
+    .. seealso:: `CookieClientIdManager`
+    """
 
     namespace = schema.ASCIILine(
-            title=_('Cookie Name'),
-            description=_(
-                "Name of cookie used to maintain state. "
-                "Must be unique to the site domain name, and only contain "
-                "ASCII letters, digits and '_'"
-                ),
-            required=True,
-            min_length=1,
-            max_length=30,
-            constraint=re.compile("^[\d\w_]+$").search,
-            )
+        title=_('Cookie Name'),
+        description=_(
+            "Name of cookie used to maintain state. "
+            "Must be unique to the site domain name, and only contain "
+            "ASCII letters, digits and '_'"
+        ),
+        required=True,
+        min_length=1,
+        max_length=30,
+        constraint=re.compile(r"^[\d\w_]+$").search,
+    )
 
     cookieLifetime = schema.Int(
-            title=_('Cookie Lifetime'),
-            description=_(
-                "Number of seconds until the browser expires the cookie. "
-                "Leave blank expire the cookie when the browser is quit. "
-                "Set to 0 to never expire. "
-                ),
-            min=0,
-            required=False,
-            default=None,
-            missing_value=None,
-            )
+        title=_('Cookie Lifetime'),
+        description=_(
+            "Number of seconds until the browser expires the cookie. "
+            "Leave blank expire the cookie when the browser is quit. "
+            "Set to 0 to never expire. "
+        ),
+        min=0,
+        required=False,
+        default=None,
+        missing_value=None,
+    )
 
     thirdparty = schema.Bool(
-            title=_('Third party cookie'),
-            description=_(
-                "Is a third party issuing the identification cookie? "
-                "Servers like Apache or Nginx have capabilities to issue "
-                "identification cookies too. If Third party cookies are "
-                "beeing used, Zope will never send a cookie back, just check "
-                "for them."
-                ),
-            required=False,
-            default=False,
-            )
+        title=_('Third party cookie'),
+        description=_(
+            "Is a third party issuing the identification cookie? "
+            "Servers like Apache or Nginx have capabilities to issue "
+            "identification cookies too. If Third party cookies are "
+            "beeing used, Zope will never send a cookie back, just check "
+            "for them."
+        ),
+        required=False,
+        default=False,
+    )
 
     domain = schema.TextLine(
-            title=_('Effective domain'),
-            description=_(
-                "An identification cookie can be restricted to a specific domain "
-                "using this option. This option sets the ``domain`` attribute "
-                "for the cookie header. It is useful for setting one "
-                "identification cookie for multiple subdomains. So if this "
-                "option is set to ``.example.org``, the cookie will be available "
-                "for subdomains like ``yourname.example.org``. "
-                "Note that if you set this option to some domain, the identification "
-                "cookie won't be available for other domains, so, for example "
-                "you won't be able to login using the SessionCredentials plugin "
-                "via another domain."
-                ),
-            required=False,
-            )
+        title=_('Effective domain'),
+        description=_(
+            "An identification cookie can be restricted to a specific domain "
+            "using this option. This option sets the ``domain`` attribute "
+            "for the cookie header. It is useful for setting one "
+            "identification cookie for multiple subdomains. So if this "
+            "option is set to ``.example.org``, the cookie will be available "
+            "for subdomains like ``yourname.example.org``. "
+            "Note that if you set this option to some domain, the identification "
+            "cookie won't be available for other domains, so, for example "
+            "you won't be able to login using the SessionCredentials plugin "
+            "via another domain."
+        ),
+        required=False,
+    )
 
     secure = schema.Bool(
         title=_('Request Secure communication'),
@@ -117,9 +130,11 @@ class ICookieClientIdManager(IClientIdManager):
         )
 
 
-@implementer(IClientIdManager, ICookieClientIdManager)
+@implementer(ICookieClientIdManager)
 class CookieClientIdManager(zope.location.Location, Persistent):
-    """Session utility implemented using cookies."""
+    """
+    Default implementation of `ICookieClientIdManager`.
+    """
 
 
     thirdparty = FieldProperty(ICookieClientIdManager['thirdparty'])
@@ -131,7 +146,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
     httpOnly = FieldProperty(ICookieClientIdManager['httpOnly'])
 
     def __init__(self, namespace=None, secret=None):
-        """Create the cookie-based cleint id manager
+        """Create the cookie-based client id manager
 
         We can pass namespace (cookie name) and/or secret string
         for generating client unique ids.
@@ -153,7 +168,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> manager2.namespace
           'service_cookie'
 
-        If we want to use CookieClientIdManager object as a non-persistent
+        If we want to use `CookieClientIdManager` object as a non-persistent
         utility, we need to specify some constant secret, so it won't be
         recreated on each application restart.
 
@@ -182,9 +197,9 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         This creates one if necessary:
 
-          >>> from io import StringIO
+          >>> from io import BytesIO
           >>> from zope.publisher.http import HTTPRequest
-          >>> request = HTTPRequest(StringIO(), {})
+          >>> request = HTTPRequest(BytesIO(), {})
           >>> bim = CookieClientIdManager()
           >>> id = bim.getClientId(request)
           >>> id == bim.getClientId(request)
@@ -192,7 +207,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         The id is retained accross requests:
 
-          >>> request2 = HTTPRequest(StringIO(), {})
+          >>> request2 = HTTPRequest(BytesIO(), {})
           >>> request2._cookies = dict(
           ...   [(name, cookie['value'])
           ...    for (name, cookie) in request.response._cookies.items()
@@ -203,8 +218,8 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           True
 
         Note that the return value of this function is a string, not
-        an IClientId. This is because this method is used to implement
-        the IClientId Adapter.
+        an `.IClientId`. This is because this method is used to implement
+        the `.IClientId` Adapter.
 
           >>> type(id) == type(u'')
           True
@@ -215,7 +230,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> request2.response._cookies
           {}
 
-        An exception to this is if the cookieLifetime is set to a
+        An exception to this is if the ``cookieLifetime`` is set to a
         non-zero integer value, in which case we do set it on every
         request, regardless of when it was last set:
 
@@ -226,11 +241,11 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> bool(request2.response._cookies)
           True
 
-        If the postOnly attribute is set to a true value, then cookies
+        If the ``postOnly`` attribute is set to a true value, then cookies
         will only be set on POST requests.
 
           >>> bim.postOnly = True
-          >>> request = HTTPRequest(StringIO(), {})
+          >>> request = HTTPRequest(BytesIO(), {})
           >>> bim.getClientId(request)
           Traceback (most recent call last):
           ...
@@ -239,7 +254,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> print(request.response.getCookie(bim.namespace))
           None
 
-          >>> request = HTTPRequest(StringIO(), {'REQUEST_METHOD': 'POST'})
+          >>> request = HTTPRequest(BytesIO(), {'REQUEST_METHOD': 'POST'})
           >>> id = bim.getClientId(request)
           >>> id == bim.getClientId(request)
           True
@@ -249,13 +264,13 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
           >>> bim.postOnly = False
 
-        It's also possible to use third-party cookies. E.g. Apache `mod_uid`
-        or Nginx `ngx_http_userid_module` are able to issue user tracking
-        cookies in front of Zope. In case thirdparty is activated Zope may
+        It's also possible to use third-party cookies. E.g. Apache ``mod_uid``
+        or Nginx ``ngx_http_userid_module`` are able to issue user tracking
+        cookies in front of Zope. In case ``thirdparty`` is activated Zope may
         not set a cookie.
 
           >>> bim.thirdparty = True
-          >>> request = HTTPRequest(StringIO(), {})
+          >>> request = HTTPRequest(BytesIO(), {})
           >>> bim.getClientId(request)
           Traceback (most recent call last):
           ...
@@ -268,13 +283,11 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         sid = self.getRequestId(request)
         if sid is None:
             if (self.thirdparty
-                or
-                (self.postOnly and not (request.method == 'POST'))
-                ):
+                    or (self.postOnly and request.method != 'POST')):
                 raise MissingClientIdException
-            else:
-                sid = self.generateUniqueId()
-                self.setRequestId(request, sid)
+
+            sid = self.generateUniqueId()
+            self.setRequestId(request, sid)
         elif (not self.thirdparty) and self.cookieLifetime:
             # If we have a finite cookie lifetime, then set the cookie
             # on each request to avoid losing it.
@@ -292,7 +305,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           True
 
         """
-        data = "%.20f%.20f%.20f" % (random.random(), time.time(), time.clock())
+        data = "%.20f%.20f%.20f" % (random.random(), time.time(), process_time())
         digest = sha1(data.encode()).digest()
         s = digestEncode(digest)
         # we store a HMAC of the random value together with it, which makes
@@ -303,13 +316,13 @@ class CookieClientIdManager(zope.location.Location, Persistent):
     def getRequestId(self, request):
         """Return the browser id encoded in request as a string
 
-        Return None if an id is not set.
+        Return `None` if an id is not set.
 
         For example:
 
-          >>> from io import StringIO
+          >>> from io import BytesIO
           >>> from zope.publisher.http import HTTPRequest
-          >>> request = HTTPRequest(StringIO(), {}, None)
+          >>> request = HTTPRequest(BytesIO(), {}, None)
           >>> bim = CookieClientIdManager()
 
         Because no cookie has been set, we get no id:
@@ -330,7 +343,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         When we set the request id, we also set a response cookie.  We
         can simulate getting this cookie back in a subsequent request:
 
-          >>> request2 = HTTPRequest(StringIO(), {}, None)
+          >>> request2 = HTTPRequest(BytesIO(), {}, None)
           >>> request2._cookies = dict(
           ...   [(name, cookie['value'])
           ...    for (name, cookie) in request.response._cookies.items()
@@ -341,20 +354,28 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> bim.getRequestId(request) == bim.getRequestId(request2)
           True
 
-        Test a corner case where Python 2.6 hmac module does not allow
-        unicode as input:
+        We allow unicode values as input, even though we work in the
+        byte-based realm of HMAC:
 
           >>> id_uni = bim.generateUniqueId()
           >>> bim.setRequestId(request, id_uni)
           >>> bim.getRequestId(request) == id_uni
           True
 
+        If the cookie data has been tampered with (doesn't correspond to our
+        secret), we will refuse to return an id:
+
+          >>> cookie = request.response.getCookie(bim.namespace)
+          >>> cookie['value'] = 'x' * len(cookie['value'])
+          >>> bim.getRequestId(request) is None
+          True
+
         If another server is managing the ClientId cookies (Apache, Nginx)
         we're returning their value without checking:
 
           >>> bim.namespace = 'uid'
           >>> bim.thirdparty = True
-          >>> request3 = HTTPRequest(StringIO(), {}, None)
+          >>> request3 = HTTPRequest(BytesIO(), {}, None)
           >>> request3._cookies = {'uid': 'AQAAf0Y4gjgAAAQ3AwMEAg=='}
           >>> bim.getRequestId(request3)
           'AQAAf0Y4gjgAAAQ3AwMEAg=='
@@ -366,41 +387,44 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         else:
             request = IHTTPApplicationRequest(request)
             sid = request.getCookies().get(self.namespace, None)
+
         if self.thirdparty:
             return sid
-        else:
 
-            # If there is an id set on the response, use that but
-            # don't trust it.  We need to check the response in case
-            # there has already been a new session created during the
-            # course of this request.
 
-            if sid is None or len(sid) != 54:
-                return None
-            s, mac = sid[:27], sid[27:]
+        # If there is an id set on the response, use that but
+        # don't trust it.  We need to check the response in case
+        # there has already been a new session created during the
+        # course of this request.
 
-            # call encode() on value s a workaround a bug where the hmac
-            # module only accepts str() types in Python 2.6
-            if (digestEncode(hmac.new(
-                    self.secret.encode(), s.encode(), digestmod=sha1
-                ).digest()).decode() != mac):
-                return None
-            else:
-                return sid
+        if sid is None or len(sid) != 54:
+            return None
+        s, mac = sid[:27], sid[27:]
+
+        # HMAC is specified to work on byte strings only so make
+        # sure to feed it that by encoding
+        mac_with_my_secret = hmac.new(self.secret.encode(), s.encode(),
+                                      digestmod=sha1).digest()
+        mac_with_my_secret = digestEncode(mac_with_my_secret).decode()
+
+        if mac_with_my_secret != mac:
+            return None
+
+        return sid
 
     def setRequestId(self, request, id):
         """Set cookie with id on request.
 
         This sets the response cookie:
 
-        See the examples in getRequestId.
+        See the examples in `getRequestId`.
 
         Note that the id is checked for validity. Setting an
         invalid value is silently ignored:
 
-            >>> from io import StringIO
+            >>> from io import BytesIO
             >>> from zope.publisher.http import HTTPRequest
-            >>> request = HTTPRequest(StringIO(), {}, None)
+            >>> request = HTTPRequest(BytesIO(), {}, None)
             >>> bim = CookieClientIdManager()
             >>> bim.getRequestId(request)
             >>> bim.setRequestId(request, 'invalid id')
@@ -420,7 +444,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         Expiry time of 0 means never (well - close enough)
 
             >>> bim.cookieLifetime = 0
-            >>> request = HTTPRequest(StringIO(), {}, None)
+            >>> request = HTTPRequest(BytesIO(), {}, None)
             >>> bid = bim.getClientId(request)
             >>> cookie = request.response.getCookie(bim.namespace)
             >>> cookie['expires']
@@ -429,7 +453,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         A non-zero value means to expire after than number of seconds:
 
             >>> bim.cookieLifetime = 3600
-            >>> request = HTTPRequest(StringIO(), {}, None)
+            >>> request = HTTPRequest(BytesIO(), {}, None)
             >>> bid = bim.getClientId(request)
             >>> cookie = request.response.getCookie(bim.namespace)
             >>> import email.utils
@@ -443,10 +467,10 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         If another server in front of Zope (Apache, Nginx) is managing the
         cookies we won't set any ClientId cookies:
 
-          >>> request = HTTPRequest(StringIO(), {}, None)
+          >>> request = HTTPRequest(BytesIO(), {}, None)
           >>> bim.thirdparty = True
           >>> from zope.testing.loggingsupport import InstalledHandler
-          >>> handler = InstalledHandler(__name__)
+          >>> handler = InstalledHandler('zope.session.http')
           >>> bim.setRequestId(request, '2345')
           >>> handler.uninstall()
           >>> len(handler.records)
@@ -459,7 +483,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
           >>> bim.thirdparty = False
           >>> bim.cookieLifetime = None
-          >>> request = HTTPRequest(StringIO(), {}, None)
+          >>> request = HTTPRequest(BytesIO(), {}, None)
           >>> bim.secure = True
           >>> bim.setRequestId(request, '1234')
           >>> from pprint import pprint
@@ -470,9 +494,9 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
           >>> bim.domain = u'.example.org'
           >>> bim.setRequestId(request, '1234')
-          >>> pprint(request.response.getCookie(bim.namespace))
-          {'domain': '.example.org', 'path': '/', 'secure': True,
-           'value': '1234'}
+          >>> cookie = request.response.getCookie(bim.namespace)
+          >>> print(cookie['domain'])
+          .example.org
 
         When the cookie is set, cache headers are added to the
         response to try to prevent the cookie header from being cached:
@@ -487,13 +511,13 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         If the httpOnly attribute is set to a true value, then the
         HttpOnly cookie option is included.
 
-          >>> request = HTTPRequest(StringIO(), {}, None)
+          >>> request = HTTPRequest(BytesIO(), {}, None)
           >>> bim.secure = False
           >>> bim.httpOnly = True
           >>> bim.setRequestId(request, '1234')
-          >>> pprint(request.response.getCookie(bim.namespace))
-          {'domain': '.example.org', 'httponly': True, 'path': '/',
-           'value': '1234'}
+          >>> cookie = request.response.getCookie(bim.namespace)
+          >>> print(cookie['httponly'])
+          True
 
         """
         # TODO: Currently, the path is the ApplicationURL. This is reasonable,
@@ -507,7 +531,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         if self.thirdparty:
             logger.warning('ClientIdManager is using thirdparty cookies, '
-                'ignoring setIdRequest call')
+                           'ignoring setIdRequest call')
             return
 
         response = request.response
@@ -540,13 +564,16 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         response.setHeader('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')
 
 def notifyVirtualHostChanged(event):
-    """Adjust cookie paths when IVirtualHostRequest information changes.
+    """
+    Adjust cookie paths when
+    `zope.publisher.interfaces.http.IVirtualHostRequest` information
+    changes.
 
-    Given an event, this method should call a CookieClientIdManager's
+    Given an event, this method should call a `CookieClientIdManager`'s
     setRequestId if a cookie is present in the response for that manager. To
     demonstrate we create a dummy manager object and event:
 
-        >>> from io import StringIO
+        >>> from io import BytesIO
         >>> @implementer(ICookieClientIdManager)
         ... class DummyManager(object):
         ...     namespace = 'foo'
@@ -559,7 +586,7 @@ def notifyVirtualHostChanged(event):
         >>> component.provideUtility(manager, IClientIdManager)
         >>> from zope.publisher.http import HTTPRequest
         >>> class DummyEvent (object):
-        ...     request = HTTPRequest(StringIO(), {}, None)
+        ...     request = HTTPRequest(BytesIO(), {}, None)
         >>> event = DummyEvent()
 
     With no cookies present, the manager should not be called:
@@ -577,7 +604,7 @@ def notifyVirtualHostChanged(event):
         'bar'
 
     If a server in front of Zope manages the ClientIds (Apache, Nginx), we
-    don't need to take care about the cookies
+    don't need to take care about the cookies:
 
         >>> manager2 = DummyManager()
         >>> manager2.thirdparty = True
@@ -592,11 +619,17 @@ def notifyVirtualHostChanged(event):
         >>> id is None
         True
 
-    Cleanup of the utility registration:
+    Of course, if there is no request associated with the event,
+    nothing happens:
+
+        >>> event2.request = None
+        >>> notifyVirtualHostChanged(event2)
+
+    .. doctest::
+        :hide:
 
         >>> import zope.component.testing
         >>> zope.component.testing.tearDown()
-
     """
     # the event sends us a IHTTPApplicationRequest, but we need a
     # IHTTPRequest for the response attribute, and so does the cookie-
@@ -604,7 +637,7 @@ def notifyVirtualHostChanged(event):
     request = IHTTPRequest(event.request, None)
     if event.request is None:
         return
-    for name, manager in component.getUtilitiesFor(IClientIdManager):
+    for _name, manager in component.getUtilitiesFor(IClientIdManager):
         if manager and ICookieClientIdManager.providedBy(manager):
             # Third party ClientId Managers need no modification at all
             if not manager.thirdparty:
