@@ -13,62 +13,51 @@
 ##############################################################################
 """Session implementation
 """
+import base64
 import random
 import time
 from heapq import heapify, heappop
 
+import persistent
+from BTrees.OOBTree import OOBTree
+
 import ZODB
 import ZODB.MappingStorage
 import zope.location
 import zope.minmax
-import persistent
-from BTrees.OOBTree import OOBTree
 
 import zope.component
 import zope.interface
-from zope.component.interfaces import ComponentLookupError
+from zope.interface.interfaces import ComponentLookupError
 from zope.publisher.interfaces import IRequest
 
 from zope.session.interfaces import \
         IClientIdManager, IClientId, ISession, ISessionDataContainer, \
         ISessionPkgData, ISessionData
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    # Py3: Now StringIO location.
-    from io import StringIO
 
 try:
     from UserDict import IterableUserDict as UserDict
 except ImportError:
-    # Py3: Now StringIO location.
+    # Py3: New UserDict location.
     from collections import UserDict
 
 try:
-    from base64 import encodebytes
-except ImportError:
-    # Py3: Python 2 only has encodestring
-    from base64 import encodestring as encodebytes
-
-try:
-    from threading import get_ident
-except ImportError:
-    # Py3: Python 2 has a different location.
     from thread import get_ident
+except ImportError:
+    # Py3: New get_ident location
+    from threading import get_ident
+
+_PY3 = bytes is not str
+encodebytes = base64.encodebytes if _PY3 else base64.encodestring
+text_type = str if _PY3 else unicode
 
 try:
-    unicode
-except NameError:
-    # Py3: Define unicode
-    unicode = str
-
-try:
-    transtable = bytes.maketrans(b'+/', b'-.')
-except AttributeError:
-    # Py3: Python 2 has a differnt location for maketrans.
     import string
     transtable = string.maketrans(b'+/', b'-.')
+except AttributeError:
+    # Python 3
+    transtable = bytes.maketrans(b'+/', b'-.')
 
 def digestEncode(s):
     """Encode SHA digest for cookie."""
@@ -77,31 +66,48 @@ def digestEncode(s):
 
 @zope.interface.implementer(IClientId)
 @zope.component.adapter(IRequest)
-class ClientId(unicode):
-    """See zope.session.interfaces.IClientId
+class ClientId(text_type):
+    """
+    Default implementation of `zope.session.interfaces.IClientId`.
 
-        >>> from zope.session import tests
-        >>> request = tests.setUp()
+    .. doctest::
+        :hide:
 
+        >>> from zope.publisher.http import HTTPRequest
+        >>> from io import BytesIO
+        >>> from zope.session.interfaces import IClientIdManager
+        >>> from zope.session.http import CookieClientIdManager
+        >>> zope.component.provideUtility(CookieClientIdManager(), IClientIdManager)
+
+    `ClientId` objects for the same request should be equal:
+
+        >>> request = HTTPRequest(BytesIO(), {}, None)
         >>> id1 = ClientId(request)
         >>> id2 = ClientId(request)
         >>> id1 == id2
         True
 
-        >>> tests.tearDown()
+    .. doctest::
+        :hide:
+
+        >>> from zope.testing import cleanup
+        >>> cleanup.tearDown()
 
     """
 
     def __new__(cls, request):
-        return unicode.__new__(cls,
-            zope.component.getUtility(IClientIdManager).getClientId(request)
-            )
+        id_manager = zope.component.getUtility(IClientIdManager)
+        cid = id_manager.getClientId(request)
+        return text_type.__new__(cls, cid)
 
 
 @zope.interface.implementer(ISessionDataContainer)
-class PersistentSessionDataContainer(
-    zope.location.Location, persistent.Persistent, UserDict):
-    """A SessionDataContainer that stores data in the ZODB"""
+class PersistentSessionDataContainer(zope.location.Location,
+                                     persistent.Persistent,
+                                     UserDict):
+    """
+    A `zope.session.interfaces.ISessionDataContainer` that stores data in the ZODB.
+    """
 
 
     _v_last_sweep = 0 # Epoch time sweep last run
@@ -114,7 +120,7 @@ class PersistentSessionDataContainer(
         self.resolution = 10 * 60
 
     def __getitem__(self, pkg_id):
-        """Retrieve an ISessionData
+        """Retrieve an `zope.session.interfaces.ISessionData`
 
             >>> sdc = PersistentSessionDataContainer()
 
@@ -164,20 +170,20 @@ class PersistentSessionDataContainer(
 
             >>> sdc.disable_implicit_sweeps = False
 
-        Ensure the lastAccessTime on the ISessionData is being updated
-        occasionally. The ISessionDataContainer maintains this whenever
-        the ISessionData is set or retrieved.
+        Ensure the ``lastAccessTime`` on the `.ISessionData` is being updated
+        occasionally. The `.ISessionDataContainer` maintains this whenever
+        the `.ISessionData` is set or retrieved.
 
-        lastAccessTime on the ISessionData is set when it is added
-        to the ISessionDataContainer
+        ``lastAccessTime`` on the ``ISessionData`` is set when it is added
+        to the ``ISessionDataContainer``
 
             >>> sdc['client_id'] = sd = SessionData()
             >>> sd.getLastAccessTime() > 0
             True
 
-        The lastAccessTime is also updated whenever the ISessionData
-        is retrieved through the ISessionDataContainer, at most
-        once every 'resolution' seconds.
+        The ``lastAccessTime`` is also updated whenever the ``ISessionData``
+        is retrieved through the ``ISessionDataContainer``, at most
+        once every ``resolution`` seconds.
 
             >>> then = sd.getLastAccessTime() - 4
             >>> sd.setLastAccessTime(then)
@@ -188,9 +194,9 @@ class PersistentSessionDataContainer(
             >>> now == sdc['client_id'].getLastAccessTime()
             True
 
-        Ensure the lastAccessTime is not modified and no garbage collection
+        Ensure the ``lastAccessTime`` is not modified and no garbage collection
         occurs when timeout == 0. We test this by faking a stale
-        ISessionData object.
+        ``ISessionData`` object.
 
             >>> sdc.timeout = 0
             >>> sd.setLastAccessTime(sd.getLastAccessTime() - 5000)
@@ -268,8 +274,8 @@ class PersistentSessionDataContainer(
             except AttributeError:
                 pass
 
-        if (self._v_last_sweep + self.resolution < now and
-            not self.disable_implicit_sweeps):
+        if (self._v_last_sweep + self.resolution < now
+                and not self.disable_implicit_sweeps):
             self.sweep()
             if getattr(self, '_v_old_sweep', None) is None:
                 self._v_old_sweep = self._v_last_sweep
@@ -283,12 +289,12 @@ class PersistentSessionDataContainer(
         return rv
 
     def __setitem__(self, pkg_id, session_data):
-        """Set an ISessionPkgData
+        """Set an `zope.session.interfaces.ISessionData`
 
             >>> sdc = PersistentSessionDataContainer()
             >>> sad = SessionData()
 
-        __setitem__ sets the ISessionData's lastAccessTime
+        ``__setitem__`` sets the ``ISessionData``'s ``lastAccessTime``
 
             >>> sad.getLastAccessTime()
             0
@@ -312,7 +318,7 @@ class PersistentSessionDataContainer(
             >>> sdc['1'] = SessionData()
             >>> sdc['2'] = SessionData()
 
-        Wind back the clock on one of the ISessionData's
+        Wind back the clock on one of the `zope.session.interfaces.ISessionData`'s
         so it gets garbage collected
 
             >>> sdc['2'].setLastAccessTime(
@@ -333,7 +339,7 @@ class PersistentSessionDataContainer(
         # calculating the expiry time to ensure that we never remove
         # data that has been accessed within timeout seconds.
         expire_time = time.time() - self.timeout - self.resolution
-        heap = [(v.getLastAccessTime(), k) for k,v in self.data.items()]
+        heap = [(v.getLastAccessTime(), k) for k, v in self.data.items()]
         heapify(heap)
         while heap:
             lastAccessTime, key = heappop(heap)
@@ -344,7 +350,8 @@ class PersistentSessionDataContainer(
 
 
 class RAMSessionDataContainer(PersistentSessionDataContainer):
-    """A SessionDataContainer that stores data in RAM.
+    """
+    A `zope.session.interfaces.ISessionDataContainer` that stores data in RAM.
 
     Currently session data is not shared between Zope clients, so
     server affinity will need to be maintained to use this in a ZEO cluster.
@@ -355,7 +362,6 @@ class RAMSessionDataContainer(PersistentSessionDataContainer):
         True
         >>> ISessionData.providedBy(sdc['1'])
         True
-
     """
     def __init__(self):
         self.resolution = 5*60
@@ -389,7 +395,9 @@ class RAMSessionDataContainer(PersistentSessionDataContainer):
 @zope.interface.implementer(ISession)
 @zope.component.adapter(IRequest)
 class Session(object):
-    """See zope.session.interfaces.ISession"""
+    """
+    Default implementation of  `zope.session.interfaces.ISession`
+    """
 
     def __init__(self, request):
         self.client_id = str(IClientId(request))
@@ -403,33 +411,52 @@ class Session(object):
             return zope.component.getUtility(ISessionDataContainer)
 
     def get(self, pkg_id, default=None):
+        """
+        Get session data.
 
-        """See zope.session.interfaces.ISession
+        .. doctest::
+            :hide:
 
-            >>> from zope.session import tests
-            >>> request = tests.setUp(PersistentSessionDataContainer)
+            >>> from zope.publisher.interfaces import IRequest
+            >>> from zope.publisher.http import HTTPRequest
+            >>> from io import BytesIO
+            >>> from zope.session.interfaces import IClientIdManager, IClientId
+            >>> from zope.session.interfaces import ISessionDataContainer
+            >>> from zope.session.http import CookieClientIdManager
+            >>> zope.component.provideUtility(CookieClientIdManager(), IClientIdManager)
+            >>> zope.component.provideAdapter(ClientId, (IRequest,), IClientId)
+            >>> sdc = PersistentSessionDataContainer()
+            >>> zope.component.provideUtility(sdc, ISessionDataContainer, '')
 
-           If we use get we get None or default returned if the pkg_id
-           is not there.
 
+        If we use `get` we get `None` or *default* returned if the *pkg_id*
+        is not there:
+
+            >>> request = HTTPRequest(BytesIO(), {}, None)
             >>> session = Session(request).get('not.there', 'default')
             >>> session
             'default'
 
-           This method is lazy and does not create the session data.
+        This method is lazy and does not create the session data:
+
             >>> session = Session(request).get('not.there')
             >>> session is None
             True
 
-           The __getitem__ method instead creates the data.
+        The ``__getitem__`` method instead creates the data:
+
             >>> session = Session(request)['not.there']
             >>> session is None
             False
             >>> session = Session(request).get('not.there')
             >>> session is None
             False
-            >>> tests.tearDown()
 
+        .. doctest::
+            :hide:
+
+            >>> import zope.testing.cleanup
+            >>> zope.testing.cleanup.tearDown()
         """
 
 
@@ -441,41 +468,51 @@ class Session(object):
             sd = sdc[self.client_id]
         except KeyError:
             return default
-        try:
-            return sd[pkg_id]
-        except KeyError:
-            return default
+
+        return sd.get(pkg_id, default)
 
 
     def __getitem__(self, pkg_id):
-        """See zope.session.interfaces.ISession
+        """
+        Get or create session data.
 
+        .. doctest::
+            :hide:
+
+            >>> from zope.publisher.interfaces import IRequest
             >>> from zope.publisher.http import HTTPRequest
-            >>> from zope.session import tests
-            >>> request = tests.setUp(PersistentSessionDataContainer)
-            >>> request2 = HTTPRequest(StringIO(''), {}, None)
+            >>> from io import BytesIO
+            >>> from zope.session.interfaces import IClientIdManager, IClientId
+            >>> from zope.session.http import CookieClientIdManager
+            >>> from zope.session.interfaces import ISessionDataContainer
+            >>> zope.component.provideUtility(CookieClientIdManager(), IClientIdManager)
+            >>> zope.component.provideAdapter(ClientId, (IRequest,), IClientId)
+            >>> sdc = PersistentSessionDataContainer()
+            >>> for product_id in ('', 'products.foo', 'products.bar'):
+            ...     zope.component.provideUtility(sdc, ISessionDataContainer, product_id)
 
+        Setup some sessions, each with a distinct namespace:
+
+            >>> request = HTTPRequest(BytesIO(), {}, None)
+            >>> request2 = HTTPRequest(BytesIO(), {}, None)
             >>> ISession.providedBy(Session(request))
             True
-
-        Setup some sessions, each with a distinct namespace
-
             >>> session1 = Session(request)['products.foo']
             >>> session2 = Session(request)['products.bar']
             >>> session3 = Session(request2)['products.bar']
 
         If we use the same parameters, we should retrieve the
-        same object
+        same object:
 
             >>> session1 is Session(request)['products.foo']
             True
 
-        Make sure it returned sane values
+        Make sure it returned sane values:
 
             >>> ISessionPkgData.providedBy(session1)
             True
 
-        Make sure that pkg_ids don't share a namespace.
+        Make sure that pkg_ids don't share a namespace:
 
             >>> session1['color'] = 'red'
             >>> session2['color'] = 'blue'
@@ -487,7 +524,11 @@ class Session(object):
             >>> session3['color']
             'vomit'
 
-            >>> tests.tearDown()
+        .. doctest::
+            :hide:
+
+            >>> from zope.testing import cleanup
+            >>> cleanup.tearDown()
 
         """
         sdc = self._sdc(pkg_id)
@@ -506,14 +547,33 @@ class Session(object):
             spd = sd[pkg_id] = SessionPkgData()
             return spd
 
+    # These methods, part of the sequence protocol, are implemented to
+    # raise exceptions. If they are not implemented and the `in`
+    # operator is used, then __getitem__ is called with integer keys
+    # until IndexError is raised...and since __getitem__ auto-creates
+    # any requested keys, that can never happen, leaving us in an
+    # infinite loop. The __contains__ method takes precedence over
+    # __iter__, but for consistency and BWC we must also not be iterable.
+    # They raise two different exceptions for BWC as well
 
     def __iter__(self):
+        # Section 5.9 of the language spec says:
+        # > for classes which do not define __contains__() but do define
+        # >__iter__() [the object is iterated]. If an exception is
+        # > raised during the iteration, it is as if in raised that exception.
+        # However, CPython turns NotImplementedError into a TypeError, but
+        # PyPy lets it propagate (like the spec says). Now that we implement
+        # __contains__, we emulate this behaviour both places.
         raise NotImplementedError
 
+    def __contains__(self, x):
+        raise TypeError
+
 
 @zope.interface.implementer(ISessionData)
 class SessionData(persistent.Persistent, UserDict):
-    """See zope.session.interfaces.ISessionData
+    """
+    Default implementation of `zope.session.interfaces.ISessionData`
 
         >>> session = SessionData()
         >>> ISessionData.providedBy(session)
@@ -521,42 +581,41 @@ class SessionData(persistent.Persistent, UserDict):
         >>> session.getLastAccessTime()
         0
 
-    Before the zope.minmax package this class used to have an attribute
-    lastAccessTime initialized in the class itself to zero.
+    Before the `zope.minmax.Maximum` object this class used to have an attribute
+    ``lastAccessTime`` initialized in the class itself to zero.
     To avoid changing the interface, that attribute has been turned into a
     property.  This part tests the behavior of a legacy session which would
     have the lastAccessTime attribute loaded from the database.
     The implementation should work for that case as well as with the new
-    session where lastAccessTime is a property.  These tests will
+    session where ``lastAccessTime`` is a property. These tests will
     be removed in a later release (see the comments in the code below).
 
-    First, create an instance of SessionData and remove a protected attribute
-    _lastAccessTime from it to make it more like the legacy SessionData.  The
-    subsequent attempt to get lastAccessTime will return a 0, because the
-    lastAccessTime is not there and the dictionary returns the default value
-    zero supplied to its get() method.
+    First, create an instance of `SessionData` and remove a protected attribute
+    ``_lastAccessTime`` from it to make it more like the legacy `SessionData`.  The
+    subsequent attempt to get ``lastAccessTime`` will return a 0, because the
+    ``lastAccessTime`` is not there and the dictionary returns the default value
+    zero supplied to its `get` method.
 
         >>> legacy_session = SessionData()
         >>> del legacy_session._lastAccessTime
         >>> legacy_session.getLastAccessTime()
         0
 
-    Now, artificially add lastAccessTime to the instance's dictionary.
-    This should make it exactly like the legacy SessionData().
+    Now, artificially add ``lastAccessTime`` to the instance's dictionary.
+    This should make it exactly like the legacy `SessionData`.
 
         >>> legacy_session.__dict__['lastAccessTime'] = 42
         >>> legacy_session.getLastAccessTime()
         42
 
-    Finally, assign to lastAccessTime.  Since the instance now looks like a
+    Finally, assign to ``lastAccessTime``. Since the instance now looks like a
     legacy instance, this will trigger, through the property mechanism, a
-    creation of a zope.minmax.Maximum() object which will take over the
+    creation of a `zope.minmax.Maximum` object which will take over the
     handling of this value and its conflict resolution from now on.
 
         >>> legacy_session.setLastAccessTime(13)
         >>> legacy_session._lastAccessTime.value
         13
-
     """
 
     # this is for support of legacy sessions; this comment and
@@ -591,11 +650,17 @@ class SessionData(persistent.Persistent, UserDict):
 
 @zope.interface.implementer(ISessionPkgData)
 class SessionPkgData(persistent.Persistent, UserDict):
-    """See zope.session.interfaces.ISessionPkgData
+    """
+    Default implementation of `zope.session.interfaces.ISessionPkgData`
 
         >>> session = SessionPkgData()
         >>> ISessionPkgData.providedBy(session)
         True
     """
+
+    # Note that this does not extend persistent.mapping.PersistentDict
+    # (which is also a UserDict subclass); we don't mark ourself
+    # modified when keys in our data are assigned/deleted
+
     def __init__(self):
         self.data = OOBTree()
